from __future__ import annotations

import traceback

from celery import states

from kinder.core.async_tasks.tasks import AsyncTask
from kinder.core.utils.address import GarConnectionException

from .executors import RepositoryExecutors
from .loggers import OutgoingMessageSmevLogLogger
from .model import FormDataMessage
from .settings import GAR_TIMEOUT_RESEND_SECONDS


class GarTimeoutFormDataResendTask(AsyncTask):
    """
    Задача повторной обработки сообщения СМЭВ FormData (AIO) при
    ошибке доступа сервиса ГАР.
    """

    description: str = __doc__

    def run(self, message, *args, **kwargs):
        super().run(*args, **kwargs)

        form_data_message = FormDataMessage(message)
        executor = RepositoryExecutors.get_executor(form_data_message)

        self.set_progress(
            progress=(
                f'Повторная обработки сообщения типа {executor.name_service}'
            )
        )

        try:
            execution_result = executor.process(form_data_message)
        except GarConnectionException as exc:
            self.set_progress(
                progress=f'{exc.get_message()}. Задача будет перезапущена '
                f'через {GAR_TIMEOUT_RESEND_SECONDS} секунд.',
                task_state=states.SUCCESS
            )
            self.apply_async(
                (message, ),
                countdown=GAR_TIMEOUT_RESEND_SECONDS,
            )
        except Exception as e:
            values = {
                'Ошибка': str(e),
                'Текст': traceback.format_exc()
            }
            self.set_progress(
                values=values,
                progress=str(e),
                task_state=states.FAILURE,
            )
        else:
            OutgoingMessageSmevLogLogger.create_log(
                form_data_message, **execution_result.logging_data)
            self.set_progress(
                progress='Завершено', task_state=states.SUCCESS)

        return self.state
